# coding=utf-8
"""
This file is part of GeoRemindMe.

GeoRemindMe is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GeoRemindMe is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GeoRemindMe.  If not, see <http://www.gnu.org/licenses/>.


"""
import calendar
from datetime import date
from itertools import groupby

from django.utils.html import conditional_escape
from django.template import Library, Node, TemplateSyntaxError

#===============================================================================
# Based on: http://journal.uggedal.com/creating-a-flexible-monthly-calendar-in-django
# Written by Eivind Uggedal - eivind@uggedal.com
#===============================================================================

class GeoCalendar(calendar.HTMLCalendar):
    def __init__(self, objects, date_field):
        super(self.__class__, self).__init__()
        self.objects = self.group_by_day(objects, date_field)
        
    def group_by_day(self, objects, date_field):
        field = lambda object : object.date_field
        return dict(
                    [(day, list(items)) for day, items in groupby(objects, field)]
                    )
    def formatmoth(self, year, month):
        self.year, self.month = year, month
        return super(self.__class__, self).formatmonth(year, month)
    
    def formatday(self, day, weekday):
        if day != 0:
            cssclass = self.cssclasses[weekday]
            if date.today() == date(self.year, self.month, day):
                cssclass += ' today'
            if day in self.objects:
                cssclass += ' filled'
                body = ['<ul>']
                for object in self.objects[day]:
                    body.append('<li>')
                    body.append('<a href="%s">' % object.get_absolute_url())
                    body.append(conditional_escape(object.title))
                    body.append('</a></li>')
                body.append('</ul>')
                return self.day_cell(cssclass, '%d %s' % (day, ''.join(body)))
            return self.day_cell(cssclass, day)
        return self.day_cell('noday', '&nbsp;')
    
import re
register = Library()

class CalendarTag(Node):
    def __init__(self, model, month, year, varname):
        exec("from georemindme.models import %s" % model)#shit code to load models in runtime :)
        self.model = eval(model)
        self.month = month
        self.year = year
        self.varname = varname
        
        
    def render(self, context):
        try:
            objects = self.model.get_by_user(context['user'])
            calendar = GeoCalendar(objects, 'starts')
            context[self.varname] = calendar.formatmonth(self.year, self.month)
        except:
            pass

        return ''

@register.tag(name="calendar")
def calendar(parser, token):
    syntax_help = "syntax: {% calendar <model> for <month> <year> as <variable> %}"
    try:
        tag_name, args = token.contents.split(None, 1)
    except:
        raise TemplateSyntaxError, "%s: without arguments, %s" % (token.contents.split()[0], syntax_help)
    
    args = re.search(r'(\w+) for (\d+) (\d+) as (\w+)', args)
    if not args:
        raise TemplateSyntaxError, "%s: wrong arguments, %s" % (token.contents.split()[0], syntax_help)
    
    return CalendarTag(*args.groups())


         
        
        

